      program hmask

! Read landmask (netCDF variable "mask_rho") and topography ("hraw"
! and "h") from a named netCDF file specified as command line argument,
! and create masked versions of both "hraw" and "h" placing them into
! a separate netCDF file. This is useful for visualization purposes.

      implicit none
      character(len=128) grid
      real(kind=8), parameter :: FillValue=1.D+33
      real(kind=8), allocatable, dimension(:,:) :: rmask, h, hraw
      integer iargc, nargs, Lm,Mm,N, ncgrd, nctarg, varid, r2dgrd(4),
     &                                      i,j, ierr, lgrd, lenstr
#include "netcdf.inc"

      nargs=iargc()
      if (nargs == 1) then
        call getarg(1,grid) ; lgrd=lenstr(grid)
      else
        write(*,'(/1x,A/ /12x,A/)') 'Usage should be: ',
     &                              'hmask grid_file.nc'
        stop
      endif

      ierr=nf_open(grid(1:lgrd), nf_nowrite, ncgrd)
      call roms_find_dims(ncgrd, grid, Lm,Mm,N)
      allocate(rmask(0:Lm+1,0:Mm+1)) ; allocate(h(0:Lm+1,0:Mm+1))
      allocate(hraw(0:Lm+1,0:Mm+1))
      call get_var_by_name_double(ncgrd, 'mask_rho', rmask)
      call get_var_by_name_double(ncgrd, 'h',        h)
      call get_var_by_name_double(ncgrd, 'hraw',     hraw)
      ierr=nf_close(ncgrd)

      do j=0,Mm+1
        do i=0,Lm+1
          if (rmask(i,j) < 0.5D0) then
            h(i,j)=FillValue ; hraw(i,j)=FillValue
          endif
        enddo
      enddo

      ierr=nf_create('hmask.nc', nf_64bit_offset, nctarg)
      if (ierr==nf_noerr) then
        ierr=nf_def_dim(nctarg, 'xi_rho',  Lm+2, r2dgrd(1))
        ierr=nf_def_dim(nctarg, 'eta_rho', Mm+2, r2dgrd(2))

        ierr=nf_def_var(nctarg, 'hraw', nf_double, 2, r2dgrd, varid)
        ierr=nf_put_att_text(nctarg,varid, 'long_name',  28,
     &                               'raw bathymetry at RHO-points')
        ierr=nf_put_att_text(nctarg,varid, 'units', 5, 'meter')
        ierr=nf_put_att_double(nctarg, varid, '_FillValue',
     &                                      nf_double, 1, FillValue)

        ierr=nf_def_var(nctarg, 'h', nf_double, 2, r2dgrd, varid)
        ierr=nf_put_att_text(nctarg,varid, 'long_name',  24,
     &                                   'bathymetry at RHO-points')
        ierr=nf_put_att_text(nctarg,varid, 'units', 5, 'meter')
        ierr=nf_put_att_double(nctarg, varid, '_FillValue',
     &                                      nf_double, 1, FillValue)
        ierr=nf_enddef(nctarg)
        call put_var_by_name_double(nctarg, 'h',    h)
        call put_var_by_name_double(nctarg, 'hraw', hraw)
        ierr=nf_close(nctarg)
      else
        write(*,'(/1x,4A/12x,A)') '### ERROR: Cannot create netCDF ',
     &                      'file ''hmask.nc''.', nf_strerror(ierr)
      endif
      end
