      program laplace

! This is merely a test program to evaluate numerical quality of
! interpolation procedures by computing discrete (Cartesian) Laplace
! of a named field and placing it into a separate netCDF file which is
! then may be viewed to evaluate discontinuities of higher-order
! derivatives. 

c--#define VERBOSE

      implicit none
      character(len=64) fname, vname
      integer ncsrc, nctarg, varid, targ_var, vtype, vdims,  vatts,
     &        unlimdimid, recdim, nrecs, dimid(8), start(8), count(8),
     &        nx,ny,nz, i,j,k, irec, ierr, iargc, lfnm,lvar, lenstr
      character(len=16) dimname(8)
      real(kind=8), dimension(:,:), allocatable :: FX,FY
      real(kind=8), dimension(:,:,:), allocatable :: A
#include "netcdf.inc"

      i=iargc()
      if (i==2) then
        call getarg(1,fname)
        call getarg(2,vname)

        lfnm=lenstr(fname)
        ierr=nf_open(fname(1:lfnm), nf_nowrite, ncsrc)
        if (ierr == nf_noerr) then
          ierr=nf_inq_unlimdim(ncsrc, unlimdimid)
          if (ierr == nf_noerr) then
            write(*,*) 'Found unlimited dimension id =', unlimdimid
          else
            unlimdimid=-1
          endif

          lvar=lenstr(vname)
          ierr=nf_inq_varid(ncsrc, vname, varid)
          if (ierr == nf_noerr) then
            ierr=nf_inq_var(ncsrc, varid, vname, vtype,
     &                             vdims,  dimid, vatts)
            if (ierr == nf_noerr) then
#ifdef VERBOSE
              write(*,'(1x,2A,4(1x,A,I3),7I3)') 'vname=',vname(1:lvar),
     &            'vtype =', vtype, 'vdims =', vdims, 'vatts =', vatts,
     &                                'dimid =', (dimid(i), i=1,vdims)
              write(*,'(1x,A)') 'dimensions:'
#endif
              nrecs=1 ; nx=1 ; ny=1; nz=1  !<-- set default values
              do i=1,vdims
                ierr=nf_inq_dim(ncsrc, dimid(i),dimname(i),count(i))
                if (ierr == nf_noerr) then
#ifdef VERBOSE
                  write(*,'(2x,I2,3(1x,A),I5)') i, 'name =', dimname(i),
     &                                             'size =',   count(i)
#endif
                  start(i)=1
                  if (dimid(i) == unlimdimid) then
                    nrecs=count(i)
                  elseif (i == 1) then      ! The variable may be two-
                    nx=count(i)             ! or three-dimensional and
                  elseif (i == 2) then      ! may or may not have time
                    ny=count(i)             ! (record) dimension which
                  elseif (i == 3) then      ! is presumed to be
                    nz=count(i)             ! unlimited.
                  endif 
                else
                  write(*,'(/1x,2A,I3,A/12x,4A/)') '### ERROR: Cannot',
     &              'determine name and size of dimension #', dimid(i),
     &              ' within netCDF', 'file ''', fname(1:lfnm), ''': ',
     &                                               nf_strerror(ierr)
                  stop
                endif
              enddo

! Create output file. It will contain only one variable and only a
! subset of the dimensions from the original file which are relevant
! for the variable.

              ierr=nf_create ('lap_'/ /vname(1:lvar)/ /'.nc',
     &                              nf_64bit_offset, nctarg)

! Define dimensions for the new file. Note that array "dimid" is 
! overwritten by this step. 

              recdim=-1
              do i=1,vdims
                if (dimid(i) == unlimdimid) then
                  ierr=nf_def_dim (nctarg, dimname(i), nf_unlimited,
     &                                                   dimid(i))
                  recdim=i
                else
                  ierr=nf_def_dim (nctarg, dimname(i), count(i),
     &                                                   dimid(i))
                endif
              enddo
              ierr=nf_def_var (nctarg, 'lap_'/ /vname(1:lvar),
     &                         nf_double, vdims,  dimid, targ_var)
              ierr=nf_enddef(nctarg)

              allocate(FX(0:nx+1,0:ny+1)) ; FX=0.D0
              allocate(FY(0:nx+1,0:ny+1)) ; FY=0.D0
              allocate(A(nx,ny,nz))

              do irec=1,nrecs
                if (recdim > 0) then
                  start(recdim)=irec ; count(recdim)=1
                endif
                ierr=nf_get_vara_double(ncsrc, varid, start,count, A)

                do k=1,nz
                  do j=1,ny
                    do i=2,nx
                      FX(i,j)=A(i,j,k)-A(i-1,j,k)
                    enddo
                  enddo
                  do j=2,ny
                    do i=1,nx
                      FY(i,j)=A(i,j,k)-A(i,j-1,k)
                    enddo
                  enddo
                  do j=1,ny
                    do i=1,nx
                      A(i,j,k)=FX(i+1,j)-FX(i,j)+FY(i,j+1)-FY(i,j)
                    enddo
                  enddo
                enddo

                ierr=nf_put_vara_double(nctarg,targ_var, start,count,A)
                if (ierr /= nf_noerr) then
                   write(*,*) '### ERROR: ', nf_strerror(ierr)
                endif
                ierr=nf_sync(nctarg)
                write(*,*) 'finished rec =', irec
              enddo  !<-- irec
              ierr=nf_close(nctarg)
            else
              write(*,'(/1x,2A,I3,1x,A/12x,4A/)') '### ERROR: Cannot ',
     &            'make general inquiry for variable with ID =', varid,
     &              'within',  'netCDF file ''', fname(1:lfnm), ''': ',
     &                                              nf_strerror(ierr)
            endif
          else
            write(*,'(/1x,4A/12x,4A/)')      '### ERROR: Cannot find ',
     &                     'netCDF variable ID for ''',  vname(1:lvar),
     &                    ''' within netCDF', 'file ''', fname(1:lfnm),
     &                                       ''': ', nf_strerror(ierr)
          endif
        else
          write(*,'(/1x,4A/12x,A/)')  '### ERROR: Cannot open netCDF ',
     &   'file ''',fname(1:lfnm),''' for reading,', nf_strerror(ierr)
        endif
      else
        write(*,'(1x,A/12x,A)') 'Usage:', 'laplace filename varname'
      endif
      end
