      program nccpstruct

! A generic tool to copy skeleton structure of netCDF file without
! filling its content.  This is a stand-alone, command-line operator
! designed to be used as
!
!          nccpstruct source.nc target.nc
!
! Basically it makes a general inquiry into "source" netCDF file and
! creates "target" netCDF file with exactly the same dimensions,
! variables and attributes.

      implicit none
      character(len=80) src_fname, trg_fname
      character(len=32) dimname, attname, varname
      integer, allocatable, dimension(:) :: dimid, vardimids
      integer nargs, ncsrc, nctarg, ndims, nvars, ngatts,
     &        unlimdimid, varid, vartype, vardims, varatts,
     &        size, i,j, ierr, lsrc, ltrg, lvar, latt, lenstr

#include "netcdf.inc"

      nargs=iargc()
      if (nargs == 2) then
        call getarg(1,src_fname) ; lsrc=lenstr(src_fname)
        call getarg(2,trg_fname) ; ltrg=lenstr(trg_fname)

        ierr=nf_open(src_fname(1:lsrc), nf_nowrite, ncsrc)
        if (ierr == nf_noerr) then
          ierr=nf_create(trg_fname(1:ltrg), nf_64bit_offset, nctarg)
          if (ierr == nf_noerr) then
            write(*,'(4x,3A)') 'Created netCDF file ''',
     &                               trg_fname(1:ltrg), '''.'
            ierr=nf_inq(ncsrc, ndims, nvars, ngatts, unlimdimid)
            if (ierr == nf_noerr) then

! Sort out dimensions:

              allocate(dimid(ndims)) ; allocate(vardimids(ndims))

              do i=1,ndims
                ierr=nf_inq_dim(ncsrc, i, dimname, size)
                if (ierr == nf_noerr) then
                  lvar=lenstr(dimname)
                  if (i == unlimdimid) size=nf_unlimited
                  ierr=nf_def_dim(nctarg, dimname(1:lvar), size,
     &                                                 dimid(i))
                  if (ierr == nf_noerr) then
                    write(*,'(6x,3A,I4)') 'Created dimension ''',
     &                              dimname(1:lvar), '''=', size
                  else
                    write(*,'(/1x,6A/12x,A/)')   '### ERROR: Cannot ',
     &                         'define dimension ''', dimname(1:lvar),
     &                      ''' in netCDF file ''', trg_fname(1:ltrg),
     &                                       '''.', nf_strerror(ierr)
                  endif
                else
                  write(*,'(/1x,2A,I3/12x,3A/12x,A/)')  '### ERROR: ',
     &                'Cannot determine name and size of dimension #',
     &                  i, 'in netCDF file ''',     src_fname(1:lsrc),
     &                                       '''.', nf_strerror(ierr)
                endif
              enddo

! Copy global attributes

              do i=1,ngatts
                ierr=nf_inq_attname(ncsrc, nf_global, i, attname)
                if (ierr == nf_noerr) then
                  latt=lenstr(attname)
                  if (attname(1:latt) == 'nc_format') then
                    write(*,'(8x,3A)') 'skipping global attribute ''',
     &                                         attname(1:latt), '''.'
                  else
                    ierr=nf_copy_att(ncsrc, nf_global, attname(1:latt),
     &                                              nctarg, nf_global)
                    if (ierr == nf_noerr) then
                      write(*,'(8x,3A)') 'Copied global attribute ''',
     &                                         attname(1:latt), '''.'
                    else
                      write(*,'(/1x,4A/12x,3A/12x,A/)')  '### ERROR: ',
     &                               'Cannot copy global attribute ''',
     &                    attname(1:latt), '''', 'into netCDF file ''',
     &                    trg_fname(1:ltrg), '''.',  nf_strerror(ierr)
                    endif
                  endif
                else
                  write(*,'(/1x,2A,I3,1x,3A/12x,A/)')   '### ERROR: ',
     &                  'Cannot determine name of global attribute #',
     &                   i, 'from netCDF file ''',  src_fname(1:lsrc),
     &                                       '''.', nf_strerror(ierr)
                endif
              enddo

! Define variables and their attributes.

              do i=1,nvars
                ierr=nf_inq_var(ncsrc, i, varname,  vartype,
     &                          vardims, vardimids, varatts)
                if (ierr == nf_noerr) then
                  lvar=lenstr(varname)
                  ierr=nf_def_var(nctarg, varname(1:lvar),  vartype,
     &                                    vardims, vardimids, varid)
                  if (ierr == nf_noerr) then
                    write(*,'(6x,3A)') 'Created variable ''',
     &                              varname(1:lvar), '''.'
                    do j=1,varatts
                      ierr=nf_inq_attname(ncsrc, i, j, attname)
                      latt=lenstr(attname)
                      ierr=nf_copy_att(ncsrc, i,  attname(1:latt),
     &                                             nctarg, varid)
                      if (ierr == nf_noerr) then
                        write(*,'(8x,3A)')    'Copied attribute ''',
     &                                         attname(1:latt), '''.'
                      else
                        write(*,'(/1x,5A/12x,3A/12x,A/)')
     &                          '### ERROR: Cannot copy attribute ''',
     &                           attname(1:latt),''' for variable ''',
     &                                    varname(1:lvar), ''' into ',
     &                          'netCDF file ''',  trg_fname(1:ltrg),
     &                                     '''.',  nf_strerror(ierr)
                      endif
                    enddo
                  else
                    write(*,'(/1x,4A/12x,3A/12x,A/)')  '### ERROR: ',
     &                  'Cannot create variable ''', varname(1:lvar),
     &                  '''', 'in netCDF file ''', trg_fname(1:ltrg),
     &                                      '''.', nf_strerror(ierr)
                  endif
                else
                  write(*,'(/1x,2A,I3/12x,4A/12x,A/)') '### ERROR: ',
     &            'Cannot make general inquiry for variable # ', i,
     &            'in ', 'netCDF file ''', src_fname(1:lsrc),  '''.',
     &                                            nf_strerror(ierr)
                endif
              enddo

              ierr=nf_enddef(nctarg)
              write(*,'(6x,3A)')  'Leaving definition mode for ''',
     &                                    trg_fname(1:ltrg), '''.'
#ifdef COPY_TIMELESS_VARS

! Copy variables which do not have unlimited dimension
! (most likely they are environmental variables)

              do i=1,nvars
                ierr=nf_inq_var(ncsrc, i, varname,  vartype,
     &                          vardims, vardimids, varatts)
                if (ierr == nf_noerr) then
                  size=1
                  do j=1,vardims
                    if (vardimids(j) == unlimdimid) then
                      size=0
                    endif
                  enddo
                  if (size > 0) then
                    lvar=lenstr(varname)
                    ierr=nf_copy_var(ncsrc, i, nctarg)
                    if (ierr == nf_noerr) then
                      write(*,'(6x,5A)')  'Copied ''', varname(1:lvar),
     &                           ''' into ''', trg_fname(1:ltrg), '''.'
                    else
                      write(*,'(/1x,6A/12x,A)')   '### ERROR: Cannot ',
     &                             'copy variable ''', varname(1:lvar),
     &                     ''' into netCDF file''',  trg_fname(1:ltrg),
     &                                        '''.', nf_strerror(ierr)
                    endif
                  endif
                else
                  write(*,'(/1x,2A,I3,1x,3A/12x,A/)')    '### ERROR: ',
     &                    'Cannot make general inquiry for variable #',
     &                     i,  'in netCDF file ''',  src_fname(1:lsrc),
     &                                        '''.', nf_strerror(ierr)
                endif
              enddo
#endif
              ierr=nf_close(nctarg)
              write(*,'(4x,3A)')  'Closed netCDF file ''',
     &                            trg_fname(1:ltrg), '''.'
              ierr=nf_close(ncsrc)
            else
               write(*,'(/1x,4A/12x,A/)')   '### ERROR: Cannot make ',
     &                          'general inquiry into netCDF file ''',
     &                    src_fname(1:lsrc), '''.', nf_strerror(ierr)
            endif
          else
            write(*,'(/1x,4A/12x,A/)')     '### ERROR: Cannot create ',
     &     'netCDF file ''',trg_fname(1:ltrg),'''.', nf_strerror(ierr)
          endif
        else
          write(*,'(/1x,4A/12x,A/)')  '### ERROR: Cannot open netCDF ',
     &                  'file ''', src_fname(1:lsrc),''' for reading.',
     &                                               nf_strerror(ierr)
        endif
      else
        write(*,'(/1x,A/ /12x,A/)')  'Usage of nccpstruct should be:',
     &                     'nccpstruct source_file.nc target_file.nc'
      endif
      end
